from django.contrib import admin

from . import models


class CertificationGroupAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'description', '_type',
        'design_id', 'accredible_id',
        'status', 'created_by', 'created',
    )
    list_filter = ('_type', 'status')
    search_fields = ('name', 'description')


class CertificationCredentialAdmin(admin.ModelAdmin):
    list_display = (
        'user', 'group', 'accredible_id', 'accredible_url', 'issued_on',
        'status', 'created',
    )
    list_filter = ('status',)


admin.site.register(models.CertificationGroup, CertificationGroupAdmin)
admin.site.register(models.CertificationCredential, CertificationCredentialAdmin)



from celery import Task
from django.apps import apps

from ..accredible import groups


class DeleteAccredibleGroupTask(Task):
    ignore_result = True
    name = 'DeleteAccredibleGroupTask'

    def run(self, group_id):
        CertificationGroup = apps.get_model('certification', 'CertificationGroup')
        group = CertificationGroup.objects.get(pk=group_id)
        groups.delete_group(group)
        group.delete()



import requests
import logging
import datetime
from dateutil import parser

from django.conf import settings

from pyaccredible.client import AccredibleWrapper

logger = logging.getLogger('accredible')


def format_date(issued_on):
    issued_on_dt = None
    if isinstance(issued_on, datetime.date) or isinstance(issued_on, datetime.datetime):
        issued_on_dt = issued_on
    else:
        issued_on_dt = parser.parse(issued_on)
    return issued_on_dt.strftime('%d %B, %Y')


def create_group(group, issued_on=None, course_name=None, design_id=None):
    client = AccredibleWrapper(
        key=settings.ACCREDIBLE_API_KEY,
        server=settings.ACCREDIBLE_SERVER_URL)
    design_id = settings.ACCREDIBLE_DESIGN.get(group._type) if not design_id else design_id

    if design_id:
        group_name = group.name
        if issued_on:
            group_name = '{} - {}'.format(group_name, format_date(issued_on))

        logger.info('GROUP: {} - {}'.format(group_name, design_id))

        response = client.group_create(
            name=group_name,
            course_name=course_name,
            course_description=group.description,
            design_id=design_id,
            attach_pdf=True,
        )

        if response.status_code == requests.codes.ok:
            group.design_id = design_id
            group.accredible_id = response.json()['group']['id']
            group.save(update_fields=['design_id', 'accredible_id', 'modified'])
        else:
            logger.error('Accredible.create_group: {}'.format(response))
            response.raise_for_status()

    else:
        logger.error('Accredible.create_group Not design: {} - {}'.format(group.name, design_id))

    return group


def delete_group(group):
    client = AccredibleWrapper(
        key=settings.ACCREDIBLE_API_KEY,
        server=settings.ACCREDIBLE_SERVER_URL)
    client.group_credentials_delete(group_id=group.accredible_id)


def update_group(group, issued_on=None, course_name=None):
    client = AccredibleWrapper(
        key=settings.ACCREDIBLE_API_KEY,
        server=settings.ACCREDIBLE_SERVER_URL)
    group_name = group.name
    if issued_on:
        group_name = '{} - {}'.format(group_name, format_date(issued_on))

    logger.info('GROUP: {} - {}'.format(group_name, group.accredible_id))

    response = client.group_update(
        group.accredible_id,
        name=group_name,
        course_name=course_name,
        course_description=group.description,
    )

    if response.status_code != requests.codes.ok:
        logger.error('Accredible.update_group: {}'.format(response))
        response.raise_for_status()



from celery import Task
from django.apps import apps

from ..accredible import credentials


class CreateAccredibleSimpleCredentialTask(Task):

    ignore_result = True
    name = 'CreateAccredibleSimpleCredentialTask'

    def run(self, credential_id):
        CertificationCredential = apps.get_model(
            'certification', 'CertificationCredential')
        credential = CertificationCredential.objects.get(pk=credential_id)
        instructor_name = credential.group.instructor_name
        credentials.create_simple_credential(credential, instructor_name)



from importlib import import_module

from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from model_utils.models import TimeStampedModel

from ..conf import settings


class CertificationCredential(TimeStampedModel):
    group = models.ForeignKey(
        'CertificationGroup', related_name='credentials',
        on_delete=models.CASCADE)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name='certifications',
        on_delete=models.CASCADE)
    accredible_id = models.IntegerField(blank=True, null=True)
    status = models.CharField(
        max_length=1,
        choices=settings.CERTIFICATION_CH_STATUS,
        default=settings.CERTIFICATION_CH_STATUS_DEFAULT)
    content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        null=True,
    )
    object_id = models.PositiveIntegerField(null=True)
    content_object = GenericForeignKey('content_type', 'object_id')
    issued_on = models.DateField(blank=True, null=True)
    accredible_url = models.URLField(blank=True, null=True)
    seo_image = models.URLField(blank=True, null=True)
    image = models.URLField(blank=True, null=True)
    badge = models.URLField(blank=True, null=True)

    def __str__(self):
        return str(self.user)

    @property
    def name(self):
        return self.group.name

    @property
    def description(self):
        return self.group.description

    @property
    def pdf(self):
        return '{}/{}.pdf'.format(settings.ACCREDIBLE_PDF_HOST, self.accredible_id)

    @property
    def user_name(self):
        try:
            return self.user.get_full_name()
        except AttributeError:
            user_name_handler = settings.ACCREDIBLE_USER_NAME_HANDLER
            module_path, function_name = user_name_handler.rsplit('.', 1)
            module = import_module(module_path)
            return getattr(module, function_name)(self.content_object)

    @property
    def user_email(self):
        try:
            return self.user.email
        except AttributeError:
            user_email_handler = settings.ACCREDIBLE_USER_EMAIL_HANDLER
            module_path, function_name = user_email_handler.rsplit('.', 1)
            module = import_module(module_path)
            return getattr(module, function_name)(self.content_object)



from celery import Task
from django.apps import apps

from ..accredible import groups, credentials


class CreateAccredibleConsultantRoleGroupTask(Task):
    ignore_result = True
    name = 'CreateAccredibleConsultantRoleGroupTask'

    def run(self, group_id, issued_on, course_name):
        CertificationGroup = apps.get_model('certification', 'CertificationGroup')
        group = CertificationGroup.objects.get(pk=group_id)
        group = groups.create_group(group, issued_on, course_name)
        credentials.create_group_credential(
            group.credentials.all(),
            group.instructor_name,
            group.accredible_id,
            issued_on=issued_on,
        )



import requests
import logging

from django.conf import settings
from django.utils import timezone

from pyaccredible.client import AccredibleWrapper

logger = logging.getLogger('accredible')


BATCH_SIZE = 10


def parse_group_response(group, data, participants_by_email):
    for data_credential in data:
        email = data_credential['recipient']['email']
        user = participants_by_email.get(email)
        credential = group.credentials.filter(user_id=user).first()
        if credential:
            parse_simple_response(credential, data_credential)


def parse_simple_response(credential, data):
    credential.accredible_id = data['id']
    credential.status = settings.CERTIFICATION_CH_STATUS_GENERATED
    credential.issued_on = data['issued_on']
    credential.accredible_url = data['url']
    credential.image = data['certificate']['image']['preview']
    credential.badge = data['badge']['image']['preview']
    credential.seo_image = data['seo_image']
    credential.save()
    generate_pdf_credential(credential)


def get_participants(credentials):
    return [(credential.user_name, credential.user_email) for credential in credentials]


def get_participants_by_email(credentials):
    return {credential.user_email: credential.user.pk for credential in credentials}


def create_group_credential(credentials, instructor_name=None, accredible_id=None, issued_on=None):
    client = AccredibleWrapper(key=settings.ACCREDIBLE_API_KEY, server=settings.ACCREDIBLE_SERVER_URL)
    group = credentials.first().group
    group_id = group.accredible_id if accredible_id is None else accredible_id
    issued_on = issued_on if issued_on is not None else timezone.now().date()

    participants = get_participants(credentials)
    participants_by_email = get_participants_by_email(credentials)

    total_credentials = len(participants)
    index = 0
    while(index < total_credentials):
        end_index = index + BATCH_SIZE
        if end_index >= total_credentials:
            end_index = total_credentials

        data = {
            'participants': participants[index:end_index],
            'group_id': group_id,
            'issued_on': issued_on,
        }
        if instructor_name:
            data['custom_attrs'] = {
                'instructor_name': instructor_name,
            }

        response = client.credential_create_bulk(**data)
        index = end_index

        if response.status_code == requests.codes.ok:
            logger.info('Accredible.credentials.create_group_credential: {}'.format(group.name))

            if 'errors' in response.json().keys():
                logger.error('Accredible.credentials.create_group_credential: {}'.format(response.json()['errors']))
            else:
                parse_group_response(
                    group, response.json()['credentials'],
                    participants_by_email)
        else:
            logger.error('Accredible.credentials.create_group_credential: {}'.format(group.name))
            response.raise_for_status()


def create_simple_credential(credential, instructor_name):
    client = AccredibleWrapper(key=settings.ACCREDIBLE_API_KEY, server=settings.ACCREDIBLE_SERVER_URL)
    data = {
        'name': credential.user_name,
        'email': credential.user_email,
        'group_id': credential.group.accredible_id,
        'issued_on': timezone.now().date(),
        'custom_attrs': {
            'instructor_name': instructor_name,
        },
    }
    response = client.credential_create(**data)
    if response.status_code == requests.codes.ok:
        parse_simple_response(credential, response.json()['credential'])
    else:
        response.raise_for_status()


def generate_pdf_credential(credential):
    client = AccredibleWrapper(key=settings.ACCREDIBLE_API_KEY, server=settings.ACCREDIBLE_SERVER_URL)
    response = client.generate_pdf(credential.accredible_id)
    if response.status_code != requests.codes.ok:
        logger.error('Accredible.credentials.generate_pdf: {}'.format(credential))
        response.raise_for_status()



    # python imports
import logging

# 3rd. libraries imports
from appconf import AppConf

# django imports
from django.conf import settings  # noqa

logger = logging.getLogger(__name__)


class CertificationConfig(AppConf):
    APP_NAME = 'certification'

    CH_STATUS_PENDING = 'A'
    CH_STATUS_GENERATED = 'B'
    CH_STATUS_DEFAULT = CH_STATUS_PENDING
    CH_STATUS = (
        (CH_STATUS_PENDING, 'Pending'),
        (CH_STATUS_GENERATED, 'Generated'),
    )

    CH_GROUP_STATUS_DEFAULT = CH_STATUS_PENDING
    CH_GROUP_STATUS = (
        (CH_STATUS_PENDING, 'Pending'),
        (CH_STATUS_GENERATED, 'Generated'),
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-10-16 10:11
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='CertificationCredential',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('accredible_id', models.IntegerField(blank=True, null=True)),
                ('status', models.CharField(choices=[('A', 'Pending'), ('B', 'Generated')], default='A', max_length=1)),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('issued_on', models.DateField(blank=True, null=True)),
                ('accredible_url', models.URLField(blank=True, null=True)),
                ('seo_image', models.URLField(blank=True, null=True)),
                ('image', models.URLField(blank=True, null=True)),
                ('badge', models.URLField(blank=True, null=True)),
                ('content_type', models.ForeignKey(null=True,
                                                   on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CertificationGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=200)),
                ('description', models.TextField(null=True)),
                ('accredible_id', models.IntegerField(blank=True, null=True)),
                ('design_id', models.IntegerField(blank=True, null=True)),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('_type', models.CharField(
                    choices=[
                        ('workshop', 'Workshop'),
                        ('consultantrolegroup', 'ConsultantRoleGroup')], max_length=100)),
                ('status', models.CharField(choices=[('A', 'Pending'), ('B', 'Generated')], default='A', max_length=1)),
                ('content_type', models.ForeignKey(null=True,
                                                   on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                                 related_name='certification_certificationgroup_related', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='certificationcredential',
            name='group',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='credentials', to='certification.CertificationGroup'),
        ),
        migrations.AddField(
            model_name='certificationcredential',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='certifications', to=settings.AUTH_USER_MODEL),
        ),
    ]



from celery import Task
from django.apps import apps

from ..accredible import groups


class UpdateAccredibleGroupTask(Task):
    ignore_result = True
    name = 'UpdateAccredibleGroupTask'

    def run(self, group_id, issued_on, course_name):
        CertificationGroup = apps.get_model('certification', 'CertificationGroup')
        group = CertificationGroup.objects.get(pk=group_id)
        groups.update_group(group, issued_on=issued_on, course_name=course_name)



from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.conf import settings

from .tasks import (
    CreateAccredibleGroupTask,
    CreateAccredibleConsultantRoleGroupTask,
    CreateAccredibleSimpleCredentialTask,
    CreateAccredibleGroupCredentialTask,
    DeleteAccredibleGroupTask,
    UpdateAccredibleGroupTask)


class CertificationGroupManager(models.Manager):

    def create_group(self, course_name, issued_on, **kwargs):

        group = self.create(**kwargs)

        if settings.ACCREDIBLE_ENABLED:
            CreateAccredibleGroupTask().s(
                group_id=group.pk,
                course_name=course_name,
                issued_on=issued_on).apply_async()

        return group

    def update_group(self, user_from, certification_group, **kwargs):
        if 'group_name' in kwargs:
            certification_group.name = kwargs['group_name']
        if 'description' in kwargs:
            certification_group.description = kwargs['description']
        certification_group.save(update_fields=['name', 'description'])

        if settings.ACCREDIBLE_ENABLED:
            UpdateAccredibleGroupTask().s(
                group_id=certification_group.pk,
                course_name=kwargs.get('course_name'),
                issued_on=kwargs.get('issued_on')).apply_async()

        return certification_group

    def remove_group(self, group):
        DeleteAccredibleGroupTask().s(
            group_id=group.pk).apply_async()

    def create_group_and_credentials(
            self, user_from, course_name, issued_on, **kwargs):
        credentials = []
        related_objects_list = kwargs.pop('related_objects_list')
        certification_group = self.create(**kwargs)

        for related_object in related_objects_list:
            credential = certification_group.credentials.create(
                user=related_object.user,
                content_type=ContentType.objects.get_for_model(related_object),
                object_id=related_object.pk)
            credentials.append(credential.pk)

        if settings.ACCREDIBLE_ENABLED and len(credentials):
            CreateAccredibleConsultantRoleGroupTask().s(
                group_id=certification_group.pk,
                issued_on=issued_on,
                course_name=course_name).apply_async()

        return certification_group

    def release_simple_credential(self, certification_group, user_from, certification_credential):

        if settings.ACCREDIBLE_ENABLED:
            CreateAccredibleSimpleCredentialTask().s(
                credential_id=certification_credential.pk).apply_async()
        return certification_credential

    def release_group_credential(self, user_from, certification_group, objects_list):

        credentials = []

        for related_object in objects_list:
            ct = ContentType.objects.get_for_model(related_object)
            credential = certification_group.credentials.create(
                user=related_object.user,
                content_type=ct,
                object_id=related_object.pk)
            credentials.append(credential.pk)

        if len(credentials) > 0 and settings.ACCREDIBLE_ENABLED:
            CreateAccredibleGroupCredentialTask().s(
                credentials_id=credentials).apply_async()



from celery import Task
from django.apps import apps

from ..accredible import credentials


class CreateAccredibleGroupCredentialTask(Task):

    ignore_result = True
    name = 'CreateAccredibleGroupCredentialTask'

    def run(self, credentials_id):
        CertificationCredential = apps.get_model('certification', 'CertificationCredential')
        user_credentials = CertificationCredential.objects.filter(id__in=credentials_id)
        instructor_name = user_credentials.first().group.instructor_name
        credentials.create_group_credential(user_credentials, instructor_name)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2019-06-27 14:04
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('certification', '0004_auto_20190524_1020'),
    ]

    operations = [
        migrations.AlterField(
            model_name='certificationgroup',
            name='_type',
            field=models.CharField(choices=[('workshop', 'Workshop'), ('consultantrole-ambassador', 'ConsultantRoleGroupAmbassador'), ('consultantrole-coach', 'ConsultantRoleGroupCoach'), ('consultantrole-trainer', 'ConsultantRoleGroupTrainer'), ('consultantrole-align-trainer', 'ConsultantRoleGroupAlignTrainer')], max_length=100),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-24 10:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('certification', '0003_auto_20181024_0932'),
    ]

    operations = [
        migrations.AlterField(
            model_name='certificationgroup',
            name='_type',
            field=models.CharField(choices=[('workshop', 'Workshop')], max_length=100),
        ),
    ]



from django.dispatch import Signal


create_certification_group = Signal(
    use_caching=True)

update_certification_group = Signal(
    providing_args=['instance', 'user_from'],
    use_caching=True)

delete_certification_group = Signal(
    providing_args=['instance'],
    use_caching=True)

create_certification_credential = Signal(
    providing_args=[
        'certification_group', 'user_from', 'certification_credential'],
    use_caching=True)

create_certification_group_credential = Signal(
    providing_args=['user_from', 'course_name', 'issued_on'],
    use_caching=True)
